/*
 * $Id: FirstStrikeAttributeResolver.java 2655 2009-11-20 13:23:46Z magicduncan $
 * $URL: https://open.jira.com/svn/IKASAN/tags/ikasan-0.8.1/samples/DemoErrorPublication/errorpub/src/main/java/org/ikasan/attributes/FirstStrikeAttributeResolver.java $
 * 
 * ====================================================================
 * Ikasan Enterprise Integration Platform
 * 
 * Distributed under the Modified BSD License.
 * Copyright notice: The copyright for this software and a full listing 
 * of individual contributors are as shown in the packaged copyright.txt 
 * file. 
 * 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *
 *  - Neither the name of the ORGANIZATION nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
package org.ikasan.attributes;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 
 * <code>AttributeResolver</code> implementation based on a List of delegate
 * <code>AttributeResolver</code>s
 * 
 * If more than one underlying resolver has a result for the candidate object
 * passed to resolveAttributes, then only the result from the first successful
 * AttributeResolver is returned
 * 
 * @author Ikasan Development Team
 * 
 */
public class FirstStrikeAttributeResolver implements AttributeResolver {

	private List<AttributeResolver> resolvers = new ArrayList<AttributeResolver>();

	public FirstStrikeAttributeResolver(List<AttributeResolver> resolvers) {
		super();
		this.resolvers = resolvers;
	}

	/* (non-Javadoc)
	 * @see org.ikasan.attributes.AttributeResolver#resolveAttributes(java.lang.Object)
	 */
	public Map<String, Object> resolveAttributes(Object object) {
		for (AttributeResolver resolver : resolvers) {
			Map<String, Object> attributes = resolver.resolveAttributes(object);
			if (!attributes.isEmpty()) {
				return attributes;
			}
		}
		return new HashMap<String, Object>();
	}

}
